==> psx4all_0.3.3.tar.gz: psx4all-0.3.3/src/maemo/sdl.cpp <==
#include "sdl.h"
#include <assert.h>

#ifndef DEPRECIATED
#define sdlscreen gp2x_sdlwrapper_screen
#endif

void	*gp2x_sdlwrapper_screen_pixels=NULL;
SDL_Surface	*gp2x_sdlwrapper_screen=NULL;
SDL_Surface *screen_real;

#ifdef RENDER_HAA
#include <SDL_haa.h>
static HAA_Actor *actor;
#endif
static int degrees = 0;
#define REAL_X 800
#define REAL_Y 480
#define BUFFER_X 480
#define BUFFER_Y 240

int gp2x_sdlwrapper_bpp=16;
unsigned short* gp2xscreen = NULL;

#define MAX_KEYS 16
unsigned long gp2x_keys[MAX_KEYS];
void gp2x_key_config();

void (*gp2x_printfchar)(gp2x_font *f, unsigned char c);
#include "font.h"

void gp2x_printfchar15(gp2x_font *f, unsigned char c)
{
  unsigned short *dst=&((unsigned short*)sdlscreen->pixels)[f->x+f->y*(sdlscreen->pitch>>1)],w,h=f->h;
//unsigned char  *src=f->data[ (c%16)*f->w + (c/16)*f->h ];
  unsigned char  *src=&f->data[c*10];

 if(f->solid)
         while(h--)
         {
          w=f->wmask;
          while(w)
          {
           if( *src & w ) *dst++=f->fg; else *dst++=f->bg;
           w>>=1;
          }
          src++;

          dst+=(sdlscreen->pitch>>1)-(f->w);
         }
 else
         while(h--)
         {
          w=f->wmask;
          while(w)
          {
           if( *src & w ) *dst=f->fg;
           dst++;
           w>>=1;
          }
          src++;

          dst+=(sdlscreen->pitch>>1)-(f->w);
         }
}

void gp2x_printf(gp2x_font *f, int x, int y, const char *format, ...)
{
 char buffer[4096]; int c; gp2x_font *g=&gp2x_default_font;
 va_list  args;

 va_start(args, format);
 vsprintf(buffer, format, args);

 if(f!=NULL) g=f;

 if(x<0) x=g->x; else g->x=x;
 if(y<0) y=g->y; else g->y=y;

 for(c=0;buffer[c];c++)
 {
  switch(buffer[c])
  {
   case '\b': g->x=x;g->y=y; break;

   case '\n': g->y+=g->h;
   case '\r': g->x=x;
              break;

   default:   gp2x_printfchar(g, (unsigned char)buffer[c]);
              g->x+=g->w;
              break;
  }
 }

 //gp2x_video_flip_single();
}

void gp2x_printf_init(gp2x_font *f, int w, int h, void *data, int fg, int bg, int solid)
{
 gp2x_printfchar=gp2x_printfchar15;
 f->x=f->y=0;
 f->wmask=1<<(w-1);
 f->w=w;
 f->h=h;
 f->data=(unsigned char *)data;
 f->fg=fg;
 f->bg=bg;
 f->solid=solid;
}

void gp2x_deinit(void)
{
#ifdef RENDER_HAA
	HAA_Quit();
#endif
	SDL_Quit();
#ifdef DEBUG
	fclose(fdbg);
#endif
}

void gp2x_init(int ticks_per_second, int bpp, int rate, int bits, int stereo, int hz, int solid_font)
{
	int res;
	SDL_Event e;
	FILE* configf = fopen(CONFIG_FILE, "r");

#ifdef DEBUG
	fdbg = fopen("debug.txt", "a+");
	fprintf(fdbg, "\n\n");
#endif

	res = SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE);
	assert(res == 0);
#ifdef RENDER_HAA
	res = HAA_Init(0);
	assert(res == 0);

	screen_real = SDL_SetVideoMode(REAL_X, REAL_Y, bpp, SDL_SWSURFACE);
	assert(screen_real);

	actor = HAA_CreateActor(SDL_SWSURFACE, BUFFER_X, BUFFER_Y, bpp);
	assert(actor);
	HAA_SetPosition(actor, 80, 0);
	HAA_SetScale(actor, 2, 2);
	HAA_Show(actor);

	gp2x_sdlwrapper_screen = actor->surface;
#else
	gp2x_sdlwrapper_screen = SDL_SetVideoMode(REAL_X, REAL_Y, bpp, SDL_DOUBLEBUF | SDL_FULLSCREEN);
#endif

	if(gp2x_sdlwrapper_screen == NULL) 
	{
		exit(0);
		return;
	}

	gp2xscreen = (unsigned short *) gp2x_sdlwrapper_screen->pixels;

	SDL_WM_SetCaption(APP_NAME, "psx4all");

	SDL_SysWMinfo      info;

	SDL_VERSION(&info.version);

	if (SDL_GetWMInfo(&info) == 1) {
		if (info.subsystem == SDL_SYSWM_X11) {

			int one = 1;
			Display *display = info.info.x11.gfxdisplay;
			Window window  = info.info.x11.wmwindow;
			
			XChangeProperty(display, window, XInternAtom (display, "_HILDON_NON_COMPOSITED_WINDOW", False), 
					XA_INTEGER, 32, PropModeReplace, (unsigned char*)&one, 1);

			Atom actions, no_trans;
			actions = XInternAtom (display, "_NET_WM_ALLOWED_ACTIONS", False);
			no_trans = XInternAtom (display, "_HILDON_WM_ACTION_NO_TRANSITIONS", False);
			XChangeProperty (display, window, actions,
					 XA_ATOM, 32, PropModeReplace,
					 (unsigned char *)&no_trans, 1);

		}
	}



	if(gp2x_sdlwrapper_screen == NULL)
	{
		exit(0);
		return;
	}
	gp2x_sdlwrapper_bpp=bpp;
	gp2x_sdlwrapper_screen_pixels=gp2x_sdlwrapper_screen->pixels;

	//SDL_EnableUNICODE(1);
	//SDL_WM_GrabInput(SDL_GRAB_ON);
	SDL_EventState(SDL_ACTIVEEVENT,SDL_IGNORE);
	SDL_EventState(SDL_MOUSEMOTION,SDL_IGNORE);
	SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_IGNORE);
	SDL_EventState(SDL_MOUSEBUTTONUP,SDL_IGNORE);
	SDL_EventState(SDL_SYSWMEVENT,SDL_IGNORE);
	SDL_EventState(SDL_VIDEORESIZE,SDL_IGNORE);
	SDL_EventState(SDL_USEREVENT,SDL_IGNORE);
	SDL_ShowCursor(SDL_DISABLE);

	//init font
	gp2x_printf_init(&gp2x_default_font,6,10,gp2x_fontf,0xFFFF,0x0000,solid_font);

	atexit(gp2x_deinit);

	if( configf == NULL )
	{
		//gp2x_key_config();
		configf = fopen(CONFIG_FILE, "w");
		if( configf )
		{
			char keyz[1024];
			sprintf(keyz, "%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u",
				gp2x_keys[0], gp2x_keys[1], gp2x_keys[2],
				gp2x_keys[3], gp2x_keys[4], gp2x_keys[5],
				gp2x_keys[6], gp2x_keys[7], gp2x_keys[8],
				gp2x_keys[9], gp2x_keys[10], gp2x_keys[11],
				gp2x_keys[12], gp2x_keys[13], gp2x_keys[14],
				gp2x_keys[15]);
			fputs(keyz, configf);
			fclose(configf);
		}
	} else {
		char keyz[1024];
		fgets(keyz, 1022, configf );
		sscanf(keyz, "%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u",
			&gp2x_keys[0], &gp2x_keys[1], &gp2x_keys[2], 
			&gp2x_keys[3], &gp2x_keys[4], &gp2x_keys[5],
			&gp2x_keys[6], &gp2x_keys[7], &gp2x_keys[8],
			&gp2x_keys[9], &gp2x_keys[10], &gp2x_keys[11],
			&gp2x_keys[12], &gp2x_keys[13], &gp2x_keys[14],
			&gp2x_keys[15]);
		fclose(configf);
	}
}

unsigned long keystate = 0;

static void keyprocess(Uint32 *st, unsigned long key, SDL_bool pressed)
{
	Uint32 val=0;
	Uint32 the_key = MAX_KEYS;
	for(int i = 0; i < MAX_KEYS; i++)
	{
		if(gp2x_keys[i] == key)
		{
			the_key = i; break;
		}
	}

	switch(the_key)
	{
		case 0:
			val=GP2X_UP; break;
		case 1:
			val=GP2X_DOWN; break;
		case 2:
			val=GP2X_LEFT; break;
		case 3:
			val=GP2X_RIGHT; break;
		case 4:
			val=GP2X_A; break;
		case 5:
			val=GP2X_B; break;
		case 6:
			val=GP2X_Y; break;
		case 7:
			val=GP2X_X; break;
		//case 8:
		//	val=GP2X_PUSH; break;
		case 9:
			val=GP2X_L; break;
		case 10:
			val=GP2X_R; break;
		case 11:
			val=GP2X_START; break;
		case 12:
			val=GP2X_SELECT; break;
		case 13:
			val=GP2X_VOL_UP; break;
		case 14:
			val=GP2X_VOL_DOWN; break;
		//case 15:
		//	val=GP2X_VOL_UP|GP2X_VOL_DOWN; break;

		default:
			return;
	}

	if (pressed)
		(*st) |= val;
	else
		(*st) &= ~val;
}

unsigned long gp2x_joystick_read(void)
{
	int res;
	static Uint32 st=0;
	SDL_Event event;

	while(SDL_PollEvent(&event))
	{
#ifdef RENDER_HAA
		if (HAA_FilterEvent(&event) == 0) continue;
#endif
		switch(event.type)
		{
			case SDL_QUIT:
				gp2x_deinit();
				exit(0);
				break;
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_BACKSPACE)
				{
					gp2x_deinit();
				}
				keyprocess(&st,event.key.keysym.scancode,SDL_TRUE);
				keyprocess(&st,event.key.keysym.unicode,SDL_TRUE);
				keyprocess(&st,event.key.keysym.sym,SDL_TRUE);
				break;
			case SDL_KEYUP:
				keyprocess(&st,event.key.keysym.scancode,SDL_FALSE);
				keyprocess(&st,event.key.keysym.unicode,SDL_FALSE);
				keyprocess(&st,event.key.keysym.sym,SDL_FALSE);
				break;
		}
	}

	keystate = st;
	return st;
}

void gp2x_key_config()
{
	SDL_Event event;

	int key_num = 0;
	while( key_num < MAX_KEYS )
	{
		if( key_num == 8 || key_num == 15 ) 
		{
			key_num++;
			continue;
		}

		switch(key_num)
		{
		case 0: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS UP TO CONTINUE"); break;
		case 1: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS DOWN TO CONTINUE"); break;
		case 2: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS LEFT TO CONTINUE"); break;
		case 3: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS RIGHT TO CONTINUE"); break;
		case 4: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON1 (SQUARE) TO CONTINUE"); break;
		case 5: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON2 (CIRCLE) TO CONTINUE"); break;
		case 6: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON3 (TRIANGLE) TO CONTINUE"); break;
		case 7: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON4 (X) TO CONTINUE"); break;
		//case 8: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS PUSH BUTTON TO CONTINUE"); break;
		case 9: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS L BUTTON TO CONTINUE"); break;
		case 10: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS R BUTTON TO CONTINUE"); break;
		case 11: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS START BUTTON TO CONTINUE"); break;
		case 12: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS SELECT BUTTON TO CONTINUE"); break;
		case 13: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS VOL UP BUTTON TO CONTINUE"); break;
		case 14: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS VOL DOWN BUTTON TO CONTINUE"); break;
		//case 15: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS VOL MIDDLE BUTTON TO CONTINUE"); break;
		}
		
		gp2x_video_flip();
		
		while(SDL_PollEvent(&event))
		{
#ifdef RENDER_HAA
			if (HAA_FilterEvent(&event) == 0) continue;
#endif
			switch(event.type)
			{
				case SDL_QUIT:
					gp2x_deinit();
					exit(0);
					break;
				case SDL_KEYDOWN:
					{
					int keysyms;
					for(keysyms = 0; keysyms < 3; keysyms++)
					{
						int the_sym;
						if( keysyms == 0) the_sym = event.key.keysym.sym;
						if( keysyms == 1) the_sym = event.key.keysym.unicode;
						if( keysyms == 2) the_sym = event.key.keysym.scancode;

						if( the_sym ) 
						{
							int key_exists = 0;
							for(int  i = 0; i < MAX_KEYS; i++)
							{
								if(gp2x_keys[i] == the_sym)
								{
									key_exists = 1;
								}
							}
							if( key_exists == 0 )
							{
								gp2x_keys[key_num] = the_sym;
								break;
							}
						}
					}
					}
					break;

				case SDL_KEYUP:
					if( event.key.keysym.sym != 0 &&  event.key.keysym.sym == gp2x_keys[key_num]) key_num++;
					else if( event.key.keysym.unicode != 0 && event.key.keysym.unicode == gp2x_keys[key_num]) key_num++;
					else if( event.key.keysym.scancode != 0 && event.key.keysym.scancode == gp2x_keys[key_num]) key_num++;
					gp2x_video_RGB_clearscreen16();
					gp2x_video_flip();
					gp2x_timer_delay(1000);
					break;
			}
		}
	}
}

void gp2x_video_RGB_clearscreen16(void)
{
  memset(sdlscreen->pixels, 0, sdlscreen->pitch*sdlscreen->h);
}

u64 gp2x_timer_read(void)
{
  struct timeval current_time;
  gettimeofday(&current_time, NULL);

  return (((u64)current_time.tv_sec * 1000000ll + current_time.tv_usec) / 1000);
}

void gp2x_video_flip()
{
	int res;
	static int x = 0;
	x++;
	if((x&0x1)) return;

#ifdef RENDER_HAA
	res = HAA_Flip(actor);
#else
	res = SDL_Flip(sdlscreen);
#endif
	assert(res == 0);
}

==> psx4all_0.3.4.tar.gz: psx4all-0.3.4/src/maemo/sdl.cpp <==
#include "sdl.h"
#include <assert.h>

#ifndef DEPRECIATED
#define sdlscreen gp2x_sdlwrapper_screen
#endif

void	*gp2x_sdlwrapper_screen_pixels=NULL;
SDL_Surface	*gp2x_sdlwrapper_screen=NULL;
SDL_Surface *screen_real;

#ifdef RENDER_HAA
#include <SDL_haa.h>
static HAA_Actor *actor;
#endif
static int degrees = 0;
#define REAL_X 800
#define REAL_Y 480
#define BUFFER_X 480
#define BUFFER_Y 240

int gp2x_sdlwrapper_bpp=16;
unsigned short* gp2xscreen = NULL;

#define MAX_KEYS 16
unsigned long gp2x_keys[MAX_KEYS];
void gp2x_key_config();

void (*gp2x_printfchar)(gp2x_font *f, unsigned char c);
#include "font.h"

void gp2x_printfchar15(gp2x_font *f, unsigned char c)
{
  unsigned short *dst=&((unsigned short*)sdlscreen->pixels)[f->x+f->y*(sdlscreen->pitch>>1)],w,h=f->h;
//unsigned char  *src=f->data[ (c%16)*f->w + (c/16)*f->h ];
  unsigned char  *src=&f->data[c*10];

 if(f->solid)
         while(h--)
         {
          w=f->wmask;
          while(w)
          {
           if( *src & w ) *dst++=f->fg; else *dst++=f->bg;
           w>>=1;
          }
          src++;

          dst+=(sdlscreen->pitch>>1)-(f->w);
         }
 else
         while(h--)
         {
          w=f->wmask;
          while(w)
          {
           if( *src & w ) *dst=f->fg;
           dst++;
           w>>=1;
          }
          src++;

          dst+=(sdlscreen->pitch>>1)-(f->w);
         }
}

void gp2x_printf(gp2x_font *f, int x, int y, const char *format, ...)
{
 char buffer[4096]; int c; gp2x_font *g=&gp2x_default_font;
 va_list  args;

 va_start(args, format);
 vsprintf(buffer, format, args);

 if(f!=NULL) g=f;

 if(x<0) x=g->x; else g->x=x;
 if(y<0) y=g->y; else g->y=y;

 for(c=0;buffer[c];c++)
 {
  switch(buffer[c])
  {
   case '\b': g->x=x;g->y=y; break;

   case '\n': g->y+=g->h;
   case '\r': g->x=x;
              break;

   default:   gp2x_printfchar(g, (unsigned char)buffer[c]);
              g->x+=g->w;
              break;
  }
 }

 //gp2x_video_flip_single();
}

void gp2x_printf_init(gp2x_font *f, int w, int h, void *data, int fg, int bg, int solid)
{
 gp2x_printfchar=gp2x_printfchar15;
 f->x=f->y=0;
 f->wmask=1<<(w-1);
 f->w=w;
 f->h=h;
 f->data=(unsigned char *)data;
 f->fg=fg;
 f->bg=bg;
 f->solid=solid;
}

void gp2x_deinit(void)
{
#ifdef RENDER_HAA
	HAA_Quit();
#endif
	SDL_Quit();
#ifdef DEBUG
	fclose(fdbg);
#endif
}

void gp2x_init(int ticks_per_second, int bpp, int rate, int bits, int stereo, int hz, int solid_font)
{
	int res;
	SDL_Event e;
	FILE* configf = fopen(CONFIG_FILE, "r");

#ifdef DEBUG
	fdbg = fopen("debug.txt", "a+");
	fprintf(fdbg, "\n\n");
#endif

	res = SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE);
	assert(res == 0);
#ifdef RENDER_HAA
	res = HAA_Init(0);
	assert(res == 0);

	screen_real = SDL_SetVideoMode(REAL_X, REAL_Y, bpp, SDL_SWSURFACE);
	assert(screen_real);

	actor = HAA_CreateActor(SDL_SWSURFACE, BUFFER_X, BUFFER_Y, bpp);
	assert(actor);
	HAA_SetPosition(actor, 80, 0);
	HAA_SetScale(actor, 2, 2);
	HAA_Show(actor);

	gp2x_sdlwrapper_screen = actor->surface;
#else
	gp2x_sdlwrapper_screen = SDL_SetVideoMode(REAL_X, REAL_Y, bpp, SDL_DOUBLEBUF | SDL_FULLSCREEN);
#endif

	if(gp2x_sdlwrapper_screen == NULL) 
	{
		exit(0);
		return;
	}

	gp2xscreen = (unsigned short *) gp2x_sdlwrapper_screen->pixels;

	SDL_WM_SetCaption(APP_NAME, "psx4all");

	SDL_SysWMinfo      info;

	SDL_VERSION(&info.version);

	if (SDL_GetWMInfo(&info) == 1) {
		if (info.subsystem == SDL_SYSWM_X11) {

			int one = 1;
			Display *display = info.info.x11.gfxdisplay;
			Window window  = info.info.x11.wmwindow;
			
			XChangeProperty(display, window, XInternAtom (display, "_HILDON_NON_COMPOSITED_WINDOW", False), 
					XA_INTEGER, 32, PropModeReplace, (unsigned char*)&one, 1);

			Atom actions, no_trans;
			actions = XInternAtom (display, "_NET_WM_ALLOWED_ACTIONS", False);
			no_trans = XInternAtom (display, "_HILDON_WM_ACTION_NO_TRANSITIONS", False);
			XChangeProperty (display, window, actions,
					 XA_ATOM, 32, PropModeReplace,
					 (unsigned char *)&no_trans, 1);

		}
	}

	Display *dpy = info.info.x11.display;
	Window win;
	if (dpy){
		win = info.info.x11.fswindow;
		if (win) XStoreName(dpy, win, APP_NAME);
		win = info.info.x11.wmwindow;
		if (win) XStoreName(dpy, win, APP_NAME);
	}

	if(gp2x_sdlwrapper_screen == NULL)
	{
		exit(0);
		return;
	}
	gp2x_sdlwrapper_bpp=bpp;
	gp2x_sdlwrapper_screen_pixels=gp2x_sdlwrapper_screen->pixels;

	//SDL_EnableUNICODE(1);
	//SDL_WM_GrabInput(SDL_GRAB_ON);
	SDL_EventState(SDL_ACTIVEEVENT,SDL_IGNORE);
	SDL_EventState(SDL_MOUSEMOTION,SDL_IGNORE);
	SDL_EventState(SDL_MOUSEBUTTONDOWN,SDL_IGNORE);
	SDL_EventState(SDL_MOUSEBUTTONUP,SDL_IGNORE);
	SDL_EventState(SDL_SYSWMEVENT,SDL_IGNORE);
	SDL_EventState(SDL_VIDEORESIZE,SDL_IGNORE);
	SDL_EventState(SDL_USEREVENT,SDL_IGNORE);
	SDL_ShowCursor(SDL_DISABLE);

	//init font
	gp2x_printf_init(&gp2x_default_font,6,10,gp2x_fontf,0xFFFF,0x0000,solid_font);

	atexit(gp2x_deinit);

	if( configf == NULL )
	{
		//gp2x_key_config();
		configf = fopen(CONFIG_FILE, "w");
		if( configf )
		{
			char keyz[1024];
			sprintf(keyz, "%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u",
				gp2x_keys[0], gp2x_keys[1], gp2x_keys[2],
				gp2x_keys[3], gp2x_keys[4], gp2x_keys[5],
				gp2x_keys[6], gp2x_keys[7], gp2x_keys[8],
				gp2x_keys[9], gp2x_keys[10], gp2x_keys[11],
				gp2x_keys[12], gp2x_keys[13], gp2x_keys[14],
				gp2x_keys[15]);
			fputs(keyz, configf);
			fclose(configf);
		}
	} else {
		char keyz[1024];
		fgets(keyz, 1022, configf );
		sscanf(keyz, "%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u:%u",
			&gp2x_keys[0], &gp2x_keys[1], &gp2x_keys[2], 
			&gp2x_keys[3], &gp2x_keys[4], &gp2x_keys[5],
			&gp2x_keys[6], &gp2x_keys[7], &gp2x_keys[8],
			&gp2x_keys[9], &gp2x_keys[10], &gp2x_keys[11],
			&gp2x_keys[12], &gp2x_keys[13], &gp2x_keys[14],
			&gp2x_keys[15]);
		fclose(configf);
	}
}

unsigned long keystate = 0;

static void keyprocess(Uint32 *st, unsigned long key, SDL_bool pressed)
{
	Uint32 val=0;
	Uint32 the_key = MAX_KEYS;
	for(int i = 0; i < MAX_KEYS; i++)
	{
		if(gp2x_keys[i] == key)
		{
			the_key = i; break;
		}
	}

	switch(the_key)
	{
		case 0:
			val=GP2X_UP; break;
		case 1:
			val=GP2X_DOWN; break;
		case 2:
			val=GP2X_LEFT; break;
		case 3:
			val=GP2X_RIGHT; break;
		case 4:
			val=GP2X_A; break;
		case 5:
			val=GP2X_B; break;
		case 6:
			val=GP2X_Y; break;
		case 7:
			val=GP2X_X; break;
		//case 8:
		//	val=GP2X_PUSH; break;
		case 9:
			val=GP2X_L; break;
		case 10:
			val=GP2X_R; break;
		case 11:
			val=GP2X_START; break;
		case 12:
			val=GP2X_SELECT; break;
		case 13:
			val=GP2X_VOL_UP; break;
		case 14:
			val=GP2X_VOL_DOWN; break;
		//case 15:
		//	val=GP2X_VOL_UP|GP2X_VOL_DOWN; break;

		default:
			return;
	}

	if (pressed)
		(*st) |= val;
	else
		(*st) &= ~val;
}

unsigned long gp2x_joystick_read(void)
{
	int res;
	static Uint32 st=0;
	SDL_Event event;

	while(SDL_PollEvent(&event))
	{
#ifdef RENDER_HAA
		if (HAA_FilterEvent(&event) == 0) continue;
#endif
		switch(event.type)
		{
			case SDL_QUIT:
				gp2x_deinit();
				exit(0);
				break;
			case SDL_KEYDOWN:
				if(event.key.keysym.sym == SDLK_BACKSPACE)
				{
					gp2x_deinit();
				}
				keyprocess(&st,event.key.keysym.scancode,SDL_TRUE);
				keyprocess(&st,event.key.keysym.unicode,SDL_TRUE);
				keyprocess(&st,event.key.keysym.sym,SDL_TRUE);
				break;
			case SDL_KEYUP:
				keyprocess(&st,event.key.keysym.scancode,SDL_FALSE);
				keyprocess(&st,event.key.keysym.unicode,SDL_FALSE);
				keyprocess(&st,event.key.keysym.sym,SDL_FALSE);
				break;
		}
	}

	keystate = st;
	return st;
}

void gp2x_key_config()
{
	SDL_Event event;

	int key_num = 0;
	while( key_num < MAX_KEYS )
	{
		if( key_num == 8 || key_num == 15 ) 
		{
			key_num++;
			continue;
		}

		switch(key_num)
		{
		case 0: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS UP TO CONTINUE"); break;
		case 1: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS DOWN TO CONTINUE"); break;
		case 2: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS LEFT TO CONTINUE"); break;
		case 3: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS RIGHT TO CONTINUE"); break;
		case 4: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON1 (SQUARE) TO CONTINUE"); break;
		case 5: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON2 (CIRCLE) TO CONTINUE"); break;
		case 6: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON3 (TRIANGLE) TO CONTINUE"); break;
		case 7: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS BUTTON4 (X) TO CONTINUE"); break;
		//case 8: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS PUSH BUTTON TO CONTINUE"); break;
		case 9: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS L BUTTON TO CONTINUE"); break;
		case 10: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS R BUTTON TO CONTINUE"); break;
		case 11: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS START BUTTON TO CONTINUE"); break;
		case 12: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS SELECT BUTTON TO CONTINUE"); break;
		case 13: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS VOL UP BUTTON TO CONTINUE"); break;
		case 14: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS VOL DOWN BUTTON TO CONTINUE"); break;
		//case 15: 	gp2x_printf(NULL, 10, 120, "KEY CONFIG: PRESS VOL MIDDLE BUTTON TO CONTINUE"); break;
		}
		
		gp2x_video_flip();
		
		while(SDL_PollEvent(&event))
		{
#ifdef RENDER_HAA
			if (HAA_FilterEvent(&event) == 0) continue;
#endif
			switch(event.type)
			{
				case SDL_QUIT:
					gp2x_deinit();
					exit(0);
					break;
				case SDL_KEYDOWN:
					{
					int keysyms;
					for(keysyms = 0; keysyms < 3; keysyms++)
					{
						int the_sym;
						if( keysyms == 0) the_sym = event.key.keysym.sym;
						if( keysyms == 1) the_sym = event.key.keysym.unicode;
						if( keysyms == 2) the_sym = event.key.keysym.scancode;

						if( the_sym ) 
						{
							int key_exists = 0;
							for(int  i = 0; i < MAX_KEYS; i++)
							{
								if(gp2x_keys[i] == the_sym)
								{
									key_exists = 1;
								}
							}
							if( key_exists == 0 )
							{
								gp2x_keys[key_num] = the_sym;
								break;
							}
						}
					}
					}
					break;

				case SDL_KEYUP:
					if( event.key.keysym.sym != 0 &&  event.key.keysym.sym == gp2x_keys[key_num]) key_num++;
					else if( event.key.keysym.unicode != 0 && event.key.keysym.unicode == gp2x_keys[key_num]) key_num++;
					else if( event.key.keysym.scancode != 0 && event.key.keysym.scancode == gp2x_keys[key_num]) key_num++;
					gp2x_video_RGB_clearscreen16();
					gp2x_video_flip();
					gp2x_timer_delay(1000);
					break;
			}
		}
	}
}

void gp2x_video_RGB_clearscreen16(void)
{
  memset(sdlscreen->pixels, 0, sdlscreen->pitch*sdlscreen->h);
}

u64 gp2x_timer_read(void)
{
  struct timeval current_time;
  gettimeofday(&current_time, NULL);

  return (((u64)current_time.tv_sec * 1000000ll + current_time.tv_usec) / 1000);
}

void gp2x_video_flip()
{
	int res;
	static int x = 0;
	x++;
	if((x&0x1)) return;

#ifdef RENDER_HAA
	res = HAA_Flip(actor);
#else
	res = SDL_Flip(sdlscreen);
#endif
	assert(res == 0);
}
